function hRD = plugin_rd()
% Reference design definition

%   Copyright 2014-2015 The MathWorks, Inc.

% Construct reference design object
hRD = hdlcoder.ReferenceDesign('SynthesisTool', 'Xilinx Vivado');

hRD.ReferenceDesignName = 'Default system (Vivado 2014.4)';
hRD.BoardName = 'ZedBoard Lianx XADC Single';

% Tool information
hRD.SupportedToolVersion = {'2014.4'};

%% Add custom design files
% add custom Vivado design
hRD.addCustomVivadoDesign( ...
    'CustomBlockDesignTcl', 'system_top.tcl', ...
    'VivadoBoardPart',      'em.avnet.com:zed:part0:1.0');
    
%% Add custom files and constraint files
hRD.CustomConstraints = {'XADC_vpvn.xdc'};
hRD.CustomFiles = {'ipcore'};

%% Add interfaces
% add clock interface
hRD.addClockInterface( ...
    'ClockConnection',   'clk_wiz_0/clk_out1', ...
    'ResetConnection',   'proc_sys_reset_0/peripheral_aresetn');

% add AXI4 and AXI4-Lite slave interfaces
hRD.addAXI4SlaveInterface( ...
    'InterfaceConnection', 'axi_interconnect_0/M00_AXI', ...
    'BaseAddress',         '0x400D0000', ...
    'MasterAddressSpace',  'processing_system7_0/Data');

hRD.DeviceTreeName = 'devicetree_axilite.dtb';

% add interface to additional IP in the reference design
hRD.addInternalIOInterface( ...
    'InterfaceID',      'xadc_temp', ...
    'InterfaceType',    'IN', ...
    'PortName',         'XADC_TEMP_OUT', ...
    'PortWidth',        16, ...
    'InterfaceConnection',  'xadc_reader_single_0/xadc_temp');
hRD.addInternalIOInterface( ...
    'InterfaceID',      'xadc_vpvn', ...
    'InterfaceType',    'IN', ...
    'PortName',         'XADC_VPVN_OUT', ...
    'PortWidth',        16, ...
    'InterfaceConnection',  'xadc_reader_single_0/xadc_vpvn');

% LocalWords:  Zynq ZC vlnv xilinx zynq zc AXI axi Addr wiz aresetn IPCORE
% LocalWords:  avnet
