function API = extractGUIParam(model)
%%EXTRACTGUIPARAM 
% The function extractGUIParam extracts all necessary information which are
% needed to initialise the GUI
%
% TODO: Check if all URL of UDP Receive Blocks allow every IP Address
% (0.0.0.0) or one specific IP
% TODO: Check if all URL of UDP Send Blocks broadcast (255.255.255.255) or
% send to one specific IP
% TODO: Check if Inports are connected to Signal, Ground or nothing and if
% so don't list them as signal

% check if system exists
if exist(model, 'file') == 4    % return value 4 for slx or mdl file
% open system
open_system(model);

API.modelName = model;
%% disable precision loss warning
warning_mode = get_param(model, 'ParameterPrecisionLossMsg');
set_param(model, 'ParameterPrecisionLossMsg', 'none');
%% get API Info of 'Lianx UDP Receive SetParameter' Block
SetParameter = find_system(model,'MaskType','LianxUDPReceiveSetParam');
if ~isempty(SetParameter)
    if numel(SetParameter) == 1
        disp('One block of type Lianx UDP Receive SetParameter found.');
        BlockName = get_param(SetParameter,'Name');
        API.SetParam.BlockName = BlockName{1};
        % get Local IP, Ports
        % Receive
        RemoteURL = get_param(SetParameter,'ReceiveremoteURL');
        API.SetParam.ReceiveRemoteURL = eval(RemoteURL{1,1});
        API.SetParam.localPort = str2double(get_param(SetParameter,'localPort'));
        % Send (Readback)
        RemoteURL = get_param(SetParameter,'SendremoteURL');
        API.SetParam.SendRemoteURL = eval(RemoteURL{1,1});
        API.SetParam.remotePort = str2double(get_param(SetParameter,'remotePort'));
        % get Signal Information (SampleRate, DataType, Name MinMax)
        % SampleTime
        SampleTime = get_param(SetParameter,'ReceiveSampleTime');
        API.SetParam.ReceiveSampleTime = eval(SampleTime{1});
        SampleTime = get_param(SetParameter,'SendSampleTime');
        API.SetParam.SendSampleTime = eval(SampleTime{1});
        % Datatype
        porthandle = get_param(SetParameter,'PortHandles');
        porthandle = porthandle{1,1}.Outport;
        simmode = get_param(model,'SimulationMode'); % get current simmode
        set_param(model,'SimulationMode','normal'); % change simmode to enable compile
        eval([model,'([],[],[],''compile'')']);
        % when in compile mode, datatype were determined and can be readout
        API.SetParam.DataType = get_param(porthandle,'CompiledPortDataType');
        eval([model,'([],[],[],''term'')']);
        set_param(model,'SimulationMode',simmode);
        % SignalName
        OutputSignalNames = get_param(SetParameter,'OutputSignalNames');
        for idx = 1:numel(porthandle)
            % check datatype of each port and warn if wrong
            if sum(strcmp(API.SetParam.DataType{idx,1},{'double','single'})) > 0
               warning('SIMULINK:wrongDataType',['Block %s\nData Type: %s invalid for Outport %i\n'...
                   'Use either Integer or Fixpoint!'],API.SetParam.BlockName,API.SetParam.DataType{idx,1},idx);
            end
            % check if ParamName exist, else write generic name
            if ~isempty(OutputSignalNames{1,1}{1,idx})
                API.SetParam.ParamName{idx,1} = OutputSignalNames{1,1}{1,idx};
            else
                API.SetParam.ParamName{idx,1} = ['No Signal Name for SetParam ' num2str(idx)];
            end
            % get defined min and max of signal
            MinMax = [get_param(SetParameter,['datatype',num2str(idx),'_min']),...
                get_param(SetParameter,['datatype',num2str(idx),'_max'])];
            API.SetParam.Min{idx,1} = eval(MinMax{1,1});
            API.SetParam.Max{idx,1} = eval(MinMax{1,2});
        end
    else
    warning('More than one Block of type Lianx UDP Receive SetParameter used! To handle multiple blocks of this type is not yet implemented.');
    end
else
    disp('No Lianx UDP Receive SetParameter Block found!')
end
%clear('MinMax','OutputSignalNames','SetParameter','porthandle','simmode','BlockName')

%% get API Info of 'Lianx UDP Send GetParameter' Block
GetParameter = find_system(model,'MaskType','LianxUDPSendGetParam');
if ~isempty(GetParameter)
    if numel(GetParameter) == 1
        disp('One block of type Lianx UDP Send GetParameter found.');
        BlockName = get_param(GetParameter,'Name');
        API.GetParam.BlockName = BlockName{1};
        % get Remote IP, Port
        RemoteURL = get_param(GetParameter,'remoteURL');
        API.GetParam.RemoteURL = eval(RemoteURL{1,1});
        API.GetParam.RemotePort = str2double(get_param(GetParameter,'remotePort'));
        % get Signal Information (SampleRate, Name)
        % SampleTime
        SampleTime = get_param(GetParameter,'OutPortSampleTime');
        API.GetParam.SampleTime = eval(SampleTime{1});
        % Datatype
        porthandle = get_param(GetParameter,'PortHandles');
        porthandle = porthandle{1,1}.Inport;
        simmode = get_param(model,'SimulationMode'); % get current simmode
        set_param(model,'SimulationMode','normal'); % change simmode to enable compile
        eval([model,'([],[],[],''compile'')']);
        % when in compile mode, datatype were determined and can be readout
        API.GetParam.DataType = get_param(porthandle,'CompiledPortDataType');
        eval([model,'([],[],[],''term'')']);
        set_param(model,'SimulationMode',simmode);
        % Signal Name
        InputSignalNames = get_param(GetParameter,'InputSignalNames');
        for idx = 1:numel(porthandle)
            % check datatype of each port and warn if wrong
            if sum(strcmp(API.GetParam.DataType{idx,1},{'double','single'})) > 0
               warning('SIMULINK:wrongDataType',['Block %s\nData Type: %s invalid for Inport %i\n'...
                   'Use either Integer or Fixpoint!'],API.GetParam.BlockName,API.GetParam.DataType{idx,1},idx);
            end
            % check if signalname exists, else write generic signal name
            if ~isempty(InputSignalNames{1,1}{1,idx})
                API.GetParam.ParamName{idx,1} = InputSignalNames{1,1}{1,idx};
            else
                API.GetParam.ParamName{idx,1} = ['No Signal Name for GetParam ' num2str(idx)];
            end
        end
    else
        warning('More than one Block of type Lianx UDP Send GetParameter used! To handle multiple blocks of this type is not yet implemented.');
    end
else
    disp('No Lianx UDP Send GetParameter Block found!')
end
%clear('GetParameter','porthandle','RemoteURL','BlockName','InputSignalNames','SampleTime');

%% get API Info of 'Lianx Control Target State Control' Block
ControlTarget = find_system(model,'MaskType','LianxControlTargetStateControl');
if ~isempty(ControlTarget)
    if numel(ControlTarget) == 1
        disp('One block of type Lianx Control Target State Control found.');
        BlockName = get_param(ControlTarget,'Name');
        API.CtrlTarget.BlockName = BlockName{1};
        % get Remote IP, Port
        % Receive
        RemoteURL = get_param(ControlTarget,'ReceiveremoteURL');
        API.CtrlTarget.ReceiveRemoteURL = eval(RemoteURL{1,1});
        API.CtrlTarget.LocalPort = str2double(get_param(ControlTarget,'localPort'));
        % Send
        RemoteURL = get_param(ControlTarget,'SendremoteURL');
        API.CtrlTarget.SendRemoteURL = eval(RemoteURL{1,1});
        API.CtrlTarget.RemotePort = str2double(get_param(ControlTarget,'remotePort'));
        % get Signal Information (SampleRate, Name)
        % SampleTime
        SampleTime = get_param(ControlTarget,'sampletime');
        API.CtrlTarget.SampleTime = eval(SampleTime{1});
        % TODO: Datatype control 
    else
        warning('More than one Block of type Lianx Control Target State Control used! To handle multiple blocks of this type is not yet implemented.');
    end
else
    disp('No Lianx Control Target State Control Block found!')
end
%clear('GetParameter','porthandle','RemoteURL','BlockName','SampleTime');

%% get API Info of 'Lianx Continuous Signal' Block
ContSig = find_system(model,'MaskType','LianxContinuousSignal');
if ~isempty(ContSig)
    if numel(ContSig) == 1
        ContSig = ContSig{1};
        disp('One block of type Lianx Continuous Signal found.');
        API.ContSig.BlockName = get_param(ContSig,'Name');
        % get Remote IP, Port
        RemoteURL = get_param(ContSig,'remoteURL');
        API.ContSig.RemoteURL = eval(RemoteURL);
        API.ContSig.RemotePort = str2double(get_param(ContSig,'remotePort'));
        % get Signal Information (SampleTime, SigBufferSize, Datatype, Name)
        % SampleTime
        API.ContSig.SampleTime = str2double(get_param(ContSig,'sampletime'));
        % SigBufferSize
        API.ContSig.SigBufferSize = str2double(get_param(ContSig,'BufferSize'));
        % Datatype
        porthandle = get_param(ContSig,'PortHandles');
        porthandle = porthandle.Inport;
        simmode = get_param(model,'SimulationMode'); % get current simmode
        set_param(model,'SimulationMode','normal'); % change simmode to enable compile
        eval([model,'([],[],[],''compile'')']);
        % when in compile mode, datatype were determined and can be readout
        API.ContSig.DataType = get_param(porthandle,'CompiledPortDataType');
        eval([model,'([],[],[],''term'')']);
        set_param(model,'SimulationMode',simmode);
        % Signal Name
        InputSignalNames = get_param(ContSig,'InputSignalNames');
        for idx = 1:numel(porthandle)
            % check datatype of each port and warn if wrong
            if sum(strcmp(API.ContSig.DataType{idx,1},{'double','single'})) > 0
               warning('SIMULINK:wrongDataType',['Block %s\nData Type: %s invalid for Inport %i\n'...
                   'Single and Double is not implemented\n'...
                   'Use either Integer or Fixpoint!'],API.ContSig.BlockName,API.ContSig.DataType{idx,1},idx);
            end
            % check if signalname exists, else write generic signal name
            if ~isempty(InputSignalNames{1,idx})
                API.ContSig.ParamName{idx,1} = InputSignalNames{1,idx};
            else
                API.ContSig.ParamName{idx,1} = ['No Signal Name for Continuous Signal ' num2str(idx)];
            end
        end
    else
        warning('More than one Block of type Lianx Continuous Signal used! To handle multiple blocks of this type is not yet implemented.');
    end
else
    disp('No Lianx Continuous Signal Block found!')
end
%clear('GetParameter','porthandle','RemoteURL','BlockName','InputSignalNames','SampleTime');

%% get API Info of LianxPLFIFOBufferPSControl Block
PLFIFOBuf = find_system(model,'MaskType','LianxPLFIFOBufferPSControl');
if ~isempty(PLFIFOBuf)
    if numel(PLFIFOBuf) == 1
        PLFIFOBuf = PLFIFOBuf{1};
        disp('One block of type Lianx PL FIFO Buffer PS Control found.');
        API.PLFIFOBuf.BlockName = get_param(PLFIFOBuf,'Name');
        % get IP and Port Infos
        % Send FIFO Samples
        RemoteURL = get_param(PLFIFOBuf, 'SendremoteURL');
        API.PLFIFOBuf.FIFORemoteURL = eval(RemoteURL);
        API.PLFIFOBuf.FIFORemotePort = str2double(get_param(PLFIFOBuf,'SendremotePort'));
        MaskWS = get_param(PLFIFOBuf,'MaskWSVariables');
        field_idx = find(strcmp({MaskWS.Name},'BufferSize')==1);
        API.PLFIFOBuf.SigBufferSize = MaskWS(field_idx).Value;
        % check SigBufferSize for integer
        if ~(round(API.PLFIFOBuf.SigBufferSize) == API.PLFIFOBuf.SigBufferSize)
            error('Block LianxPLFIFOBufferPSControl calculated Signal BufferSize wrong, check Algorithm!');
        end
        % Receive Param
        RemoteURL = get_param(PLFIFOBuf,'ReceiveremoteURL');
        API.PLFIFOBuf.RemoteURL = eval(RemoteURL);
        API.PLFIFOBuf.LocalPort = str2double(get_param(PLFIFOBuf,'ReceivelocalPort'));
        % Send Param
        RemoteURL = get_param(PLFIFOBuf,'CurrentSetValuesremoteURL');
        API.PLFIFOBuf.SendRemoteURL = eval(RemoteURL);
        API.PLFIFOBuf.RemotePort = str2double(get_param(PLFIFOBuf,'CurrentSetValuesremotePort'));
        % get Signal Information (SampleTime, SigBufferSize, Datatype, Name)
        % SampleTime (Param Send Receive
        API.PLFIFOBuf.SampleTime = str2double(get_param(PLFIFOBuf,'SetSampleTime'));
        % FIFO Send Sample Time
        API.PLFIFOBuf.FIFOPopSampleTime = str2double(get_param(PLFIFOBuf,'FIFOPopSampleTime'));
        % FIFO Sample Number
        API.PLFIFOBuf.PLFIFOSize = get_param(PLFIFOBuf,'PL_FIFO_Size');
        if ischar(API.PLFIFOBuf.PLFIFOSize)
            modelworkspace = get_param(model,'ModelWorkspace');
            %strcmp(API.PLFIFOBuf.PLFIFOSize,'PL_FIFO_Size')
            if hasVariable(modelworkspace, API.PLFIFOBuf.PLFIFOSize)
                FIFOSize = getVariable(modelworkspace,API.PLFIFOBuf.PLFIFOSize);
                API.PLFIFOBuf.PLFIFOSize = FIFOSize.Value;
            end
        elseif isnum(API.PLFIFOBuf.PLFIFOSize)
            warning('Check if PL FIFO Size is the same as in the PL Part!')
        else
            error('Invalid Value for PL FIFO Size')
        end
        % Datatype for Inport 1:2
        inport_idx = 1:2;
        outport_idx = 1:6;
        porthandle = get_param(PLFIFOBuf,'PortHandles');
        simmode = get_param(model,'SimulationMode'); % get current simmode
        set_param(model,'SimulationMode','normal'); % change simmode to enable compile
        eval([model,'([],[],[],''compile'')']);
        % when in compile mode, datatype were determined and can be readout
        DataTypeInport = get_param(porthandle.Inport,'CompiledPortDataType');
        DataTypeOutport = get_param(porthandle.Outport,'CompiledPortDataType');
        API.PLFIFOBuf.InportDataType = DataTypeInport(inport_idx);
        API.PLFIFOBuf.OutportDataType = DataTypeOutport(outport_idx);
        eval([model,'([],[],[],''term'')']);
        set_param(model,'SimulationMode',simmode);
        % Input Signal Name
        InputSignalNames = get_param(PLFIFOBuf,'InputSignalNames');
        for idx = 1:2 % 1:2 because of only two input parameters to transmitt
            % check datatype of each port and warn if wrong
            if sum(strcmp(API.PLFIFOBuf.InportDataType{idx,1},{'double','single'})) > 0
               warning('SIMULINK:wrongDataType',['Block %s\nData Type: %s invalid for Inport %i\n'...
                   'Single and Double is not implemented\n'...
                   'Use either Integer or Fixpoint!'],API.PLFIFOBuf.BlockName,API.PLFIFOBuf.DataType{idx,1},idx);
            end
            % check if signalname exists, else write generic signal name
            if ~isempty(InputSignalNames{1,idx})
                API.PLFIFOBuf.GetParamName{idx,1} = InputSignalNames{1,idx};
            else
                API.PLFIFOBuf.GetParamName{idx,1} = ['No Signal Name for Get Trigger Signal ' num2str(idx)];
            end
        end
        % Output Signal Name and Datatype
        OutputSignalNames = get_param(PLFIFOBuf,'OutputSignalNames');
        for idx = 1:2 % 1:2 only this are important for signalname
            % check datatype of each port and warn if wrong
            if sum(strcmp(API.PLFIFOBuf.OutportDataType{idx,1},{'double','single'})) > 0
               warning('SIMULINK:wrongDataType',['Block %s\nData Type: %s invalid for Inport %i\n'...
                   'Single and Double is not implemented\n'...
                   'Use either Integer or Fixpoint!'],API.PLFIFOBuf.BlockName,API.PLFIFOBuf.DataType{idx,1},idx);
            end
            % check if signalname exists, else write generic signal name
            if ~isempty(OutputSignalNames{1,idx})
                API.PLFIFOBuf.SetParamName{idx,1} = OutputSignalNames{1,idx};
            else
                API.PLFIFOBuf.SetParamName{idx,1} = ['No Signal Name for Set Trigger Signal ' num2str(idx)];
            end
            % get defined min and max of signal
            Min = get_param(PLFIFOBuf,['datatype',num2str(idx),'_min']);
            Max = get_param(PLFIFOBuf,['datatype',num2str(idx),'_max']);
            API.PLFIFOBuf.Min{idx,1} = eval(Min);
            API.PLFIFOBuf.Max{idx,1} = eval(Max);
        end
            
    else
        warning('More than one Block of type Lianx PL FIFO Buffer PS Control used! To handle multiple blocks of this type is not yet implemented.');
    end
else
    disp('No Lianx PL FIFO Buffer PS Control Block found!')
end
%% enable precision warning
set_param(model, 'ParameterPrecisionLossMsg',warning_mode);
else
    error('This model does not exist!')
end

