function [ handles, state ] = udp_ContSig( input, on_off )
%UDP_CONTSIG Summary of this function goes here
%   Detailed explanation goes here
handles = input;
handles.xAxisTimeVal = str2double(get(handles.xAxisTime,'String'));

try
    if on_off   % setup udp object
        % configure graphic
        handles.ringBufferSize = 10000*handles.API.ContSig.SigBufferSize;
        for idx = 1:numel(handles.API.ContSig.ParamName)
            ylim = get(handles.(['ContinuousSignal',num2str(idx)]),'YLim');
            % plot Version 1: Datatips are available, is slower than
            % animatedline
            %handles.h(idx) = plot(handles.(['ContinuousSignal', num2str(idx)]),NaN, NaN);
            % plot Version 2: Datatips are not available, is very fast
            c = get(handles.(['ContinuousSignal',num2str(idx)]), 'Children');
            delete(c(1:end));
            handles.h(idx) = animatedline('Parent',handles.(['ContinuousSignal', num2str(idx)]));
            
            %(it is important to have manual limits to speed up draw performance http://undocumentedmatlab.com/blog/plot-performance
            set(handles.(['ContinuousSignal',num2str(idx)]),...
               'xlim', [0, handles.xAxisTimeVal], 'ylim', ylim,... 
               'XLimMode','manual','YLimMode','manual',...
               'XGrid','on', 'YGrid','on', 'XMinorGrid','on', 'YMinorGrid','on',...
               'FontSize',8);
            handles.ContSig.t(idx,:) = (((idx-1)*handles.API.ContSig.SigBufferSize)+1):(((idx-1)*handles.API.ContSig.SigBufferSize)+ handles.API.ContSig.SigBufferSize);
            %legend(handles.h(idx),handles.API.ContSig.ParamName(idx),'Interpreter','none','Location','northwest');
            % legend makes callback very slow (rendering and positioning)
            % approx. 30ms per cycle!
            % set a substitude for legend
            handles.(['ContinuousSignal',num2str(idx),'Signalname']).String = handles.API.ContSig.ParamName(idx);
            oldpos = get(handles.(['ContinuousSignal',num2str(idx),'Signalname']),'Position');
            extent = get(handles.(['ContinuousSignal',num2str(idx),'Signalname']),'Extent');
            set(handles.(['ContinuousSignal',num2str(idx),'Signalname']),'Position',[oldpos(1:2), extent(3:4)]);
        end
        % clear all animatedline objects
        
        % xAxes Label
        ax = get(handles.ContinuousSignal2,'XLabel');
        ax.String = 'time in s';
        
        %set(handles.xAxisTime,'Enable','off')   % no change to x Axis Time
        % precalculate plot parameter
        handles.ContSig.ReadoutSize = handles.API.ContSig.SigBufferSize*numel(handles.API.ContSig.ParamName);
        
        % allocate data storage
        setappdata(handles.GUI_Storage,'SignalData',nan(handles.ringBufferSize,4));
        
        % TODO: GUI create entry to define location of data logging
        handles.StorePath = 'StoredData';
        if exist(handles.StorePath,'dir') ~= 7
        mkdir(handles.StorePath);
        end
        
        % create Datatype object
        handles.ContSig.T(1) = numerictype(handles.API.ContSig.DataType{1});
        handles.ContSig.T(1).WordLength = 32;
        handles.ContSig.T(2) = numerictype(handles.API.ContSig.DataType{2});
        handles.ContSig.T(2).WordLength = 32;
        
        
        % configure UDP obj
        IPAddress = get(handles.TargetIPAddress,'String');
        remotePort = 27000;
        localPort = handles.API.ContSig.RemotePort;

        % create udp object
        handles.udp.ContSig = udp(IPAddress, remotePort, 'LocalPort',localPort);
        handles.udp.ContSig.Name = 'udp_ContSig';
        handles.udp.ContSig.ByteOrder = 'littleEndian';
        handles.udp.ContSig.BytesAvailableFcnCount = ...
            4*numel(handles.API.ContSig.ParamName)*handles.API.ContSig.SigBufferSize;   % times 4 due to the datatype uint32 =  4bytes
        handles.udp.ContSig.BytesAvailableFcnMode = 'byte';
        handles.udp.ContSig.BytesAvailableFcn = {'update_ContSig_Callback', handles};
        handles.udp.ContSig.InputBufferSize = 8*handles.udp.ContSig.BytesAvailableFcnCount;
        handles.udp.ContSig.DatagramTerminateMode = 'off';

        fopen(handles.udp.ContSig);
        state = 1;
        
    elseif on_off == 0        % delete udp object
        fclose(handles.udp.ContSig);
        % load remaining values
        data = getappdata(handles.GUI_Storage,'SignalData');
        % determine last stored Value
        ValuesPerChannel = handles.udp.ContSig.ValuesReceived/2;
        lastValue = mod(ValuesPerChannel,handles.ringBufferSize);
        remainingValues = data(1:lastValue,:);
        % save remaining Values as .mat
        t = datestr(now,'yy_mm_dd_HHMMss'); % generate time stamp
%         fid = fopen(fullfile(handles.StorePath,sprintf('%s_ContinuousSignal_remaining.bin',t)),'w');
%         fwrite(fid,remainingValues,'single');
%         fclose(fid);
        save(fullfile(handles.StorePath,sprintf('%s_ContinuousSignal_remaining.mat',t)),'remainingValues');
        % delete udp object
        delete(handles.udp.ContSig);
        state = 0;
    else
        error('Wrong input value on_off');
    end
catch
    %set(handles.xAxisTime,'Enable','on');
    try
        delete(handles.udp.ContSig);
    catch
    end
    state = -1;
end




end

