function [ handles, state ] = udp_Trigger( input, on_off )
%UDP_TRIGGERSETVALUES Summary of this function goes here
%   Detailed explanation goes here
handles = input;
PushSampleTime = str2double(get(handles.FIFOPushSampleTime,'String'))*10*10^(-9);
xlimit = [0, (handles.API.PLFIFOBuf.PLFIFOSize)*PushSampleTime];

try
    if on_off   % setup udp object
        SigBufferSize = handles.API.PLFIFOBuf.SigBufferSize;
        idx_max = handles.API.PLFIFOBuf.PLFIFOSize * numel(handles.API.PLFIFOBuf.GetParamName) / SigBufferSize;
        base_Vector = @(idx) [idx*SigBufferSize + 1:1:(idx*SigBufferSize+SigBufferSize)];

        % configure graphic
        for idx = 1:numel(handles.API.PLFIFOBuf.GetParamName)
            % plot Version 1:
            %handles.hFIFO(idx) = plot(handles.(['TriggerData', num2str(idx)]),NaN, NaN);
            % plot Version 2:
            c = get(handles.(['TriggerData',num2str(idx)]), 'Children');
            delete(c(1:end));
            handles.hFIFO(idx) = animatedline('Parent',handles.(['TriggerData', num2str(idx)]));
            set(handles.(['TriggerData',num2str(idx)]), 'xlim', xlimit, ...%'ylim', ylimit(idx,:),...
                'XLimMode','manual', 'YLimMode','manual',...
                'XGrid','on', 'YGrid','on', 'XMinorGrid','on', 'YMinorGrid','on',...
                'FontSize',8);
            %legend(handles.hFIFO(idx),handles.API.PLFIFOBuf.GetParamName(idx),'Interpreter','none','Location','NorthWest');
            % set a substitude for legend
            handles.(['TriggerData',num2str(idx),'Signalname']).String = handles.API.PLFIFOBuf.GetParamName(idx);
            oldpos = get(handles.(['TriggerData',num2str(idx),'Signalname']),'Position');
            extent = get(handles.(['TriggerData',num2str(idx),'Signalname']),'Extent');
            set(handles.(['TriggerData',num2str(idx),'Signalname']),'Position',[oldpos(1:2),extent(3:4)]);
            
            % readout vector
            p = idx-1;
            vec_idx = 0+p:2:idx_max-1+p;
            for n = 1:numel(vec_idx)
                handles.PLFIFOGet.t(idx,(n-1)*SigBufferSize+1:(n-1)*SigBufferSize+SigBufferSize) =  base_Vector(vec_idx(n));
            end
            %handles.PLFIFOGet.t(idx,:) = (((idx-1)*handles.API.PLFIFOBuf.SigBufferSize)+1):(((idx-1)*handles.API.PLFIFOBuf.SigBufferSize)+ handles.API.PLFIFOBuf.SigBufferSize);
        end
        
        
        % create Datatype object
        handles.PLFIFOGet.T(1) = numerictype(handles.API.PLFIFOBuf.InportDataType{1});
        handles.PLFIFOGet.T(1).WordLength = 32;
        handles.PLFIFOGet.T(2) = numerictype(handles.API.PLFIFOBuf.InportDataType{2});
        handles.PLFIFOGet.T(2).WordLength = 32;
        
        % allocate data storage
        setappdata(handles.GUI_Storage,'TriggerData',nan(handles.API.PLFIFOBuf.PLFIFOSize,4));
        
        % TODO: GUI create entry to define location of data logging
        handles.TriggerStorePath = 'TriggerData';
        if exist(handles.TriggerStorePath,'dir') ~= 7
        mkdir(handles.TriggerStorePath);
        end
        
        handles.PLFIFOGet.ReadoutSizeGetParam = handles.API.PLFIFOBuf.PLFIFOSize*numel(handles.API.PLFIFOBuf.GetParamName);%handles.API.PLFIFOBuf.SigBufferSize*numel(handles.API.PLFIFOBuf.GetParamName);
        handles.PLFIFOGet.FIFOPushSampleTime = PushSampleTime;
        % configure UDP obj (FIFO Set, Readback Set)
        IPAddress = get(handles.TargetIPAddress,'String');
        remotePort = 28002;
        localPort = handles.API.PLFIFOBuf.FIFORemotePort;

        % create udp object if it does not exist
        a = instrfind('Name','udp_Trigger');
        if ~isempty(a)
            disp('Object allready exist');
            fclose(a);
        else
        handles.udp.PLFIFOGet = udp(IPAddress, remotePort, 'LocalPort',localPort);
        end
        handles.udp.PLFIFOGet.Name = 'udp_Trigger';
        handles.udp.PLFIFOGet.ByteOrder = 'littleEndian';
        %-------------
        % prepare rescue function due to problem with automatically
        % update_Trigger_Callback
%         t = timerfind('Name','RescueTimer');
%         if isempty(t)
%             handles.rescue_t = timer('TimerFcn',{@rescue_Trigger_Callback, handles},'StartDelay',3);
%             handles.rescue_t.Name = 'RescueTimer';
%         else
%         end
        %-------------
        handles.udp.PLFIFOGet.BytesAvailableFcnCount = ...
            4*handles.PLFIFOGet.ReadoutSizeGetParam;
            %4*handles.PLFIFOGet.ReadoutSizeGetParam;   % times 4 due to the datatype uint32 =  4bytes
        handles.udp.PLFIFOGet.BytesAvailableFcnMode = 'byte';
        handles.udp.PLFIFOGet.BytesAvailableFcn = {'update_Trigger_Callback', handles};
        handles.udp.PLFIFOGet.InputBufferSize = ...
            4*handles.API.PLFIFOBuf.PLFIFOSize*numel(handles.API.PLFIFOBuf.GetParamName); % max input buffer size
        handles.udp.PLFIFOGet.DatagramTerminateMode = 'off';

        % prepare Send Values
        SendVector = handles.lastSendVector;
        TriggerPort_idx = 4;
        if handles.CurrentTriggerValue.Value == 1
            SendVector(TriggerPort_idx) = 0;
        elseif handles.CurrentTriggerValue.Value == 0
            SendVector(TriggerPort_idx) = 1;
        else
            error('Current Trigger Value is invalid!');
        end
        handles.SignalTrigger.Enable = 'off';
        handles.TriggerReadout.BackgroundColor = [1,0,0];
        
        
        % open udp object
        fopen(handles.udp.PLFIFOGet);

        % send data over other udp object to start triggering
        fwrite(handles.udp.PLFIFOBuf,SendVector,'uint32');
        % start Watchdog
%         start(handles.rescue_t);
        handles.lastSendVector = SendVector;
        state = 1;
        
    elseif on_off == 0        % delete udp object
        fclose(handles.udp.PLFIFOGet);
        % delete udp object
        %delete(handles.udp.PLFIFOGet);
        state = 0;
    else
        error('Wrong input value on_off');
    end
catch
    try
        try
        fclose(handles.udp.PLFIFOGet);
        catch
        end
        delete(handles.udp.PLFIFOGet);
    catch
    end
    state = -1;
end

end

