function hRD = plugin_rd()
% Reference design definition

%   Copyright 2014-2015 The MathWorks, Inc.

% Construct reference design object
hRD = hdlcoder.ReferenceDesign('SynthesisTool', 'Xilinx Vivado');

hRD.ReferenceDesignName = 'Default system (Vivado 2014.4)';
hRD.BoardName = 'XADC Demo';

% Tool information
hRD.SupportedToolVersion = {'2014.4'};

%% Add custom design files
% add custom Vivado design
hRD.addCustomVivadoDesign( ...
    'CustomBlockDesignTcl', 'system_top.tcl', ...
    'VivadoBoardPart',      'em.avnet.com:zed:part0:1.0');

%% Add custom files and constraint files
hRD.CustomConstraints = {'XADC_vpvn_vaux0_vaux8.xdc'};
hRD.CustomFiles = {'ipcore'};

%% Add interfaces
% add clock interface
hRD.addClockInterface( ...
    'ClockConnection',   'clk_wiz_0/clk_out1', ...
    'ResetConnection',   'proc_sys_reset_0/peripheral_aresetn');

% add AXI4 and AXI4-Lite slave interfaces
hRD.addAXI4SlaveInterface( ...
    'InterfaceConnection', 'axi_interconnect_0/M00_AXI', ...
    'BaseAddress',         '0x400D0000', ...
    'MasterAddressSpace',  'processing_system7_0/Data');

hRD.DeviceTreeName = 'devicetree_axilite.dtb';

% add interface to additional IP in the reference design
hRD.addInternalIOInterface( ...
    'InterfaceID',      'Output data bus (XADC DRP)', ...
    'InterfaceType',    'IN', ...
    'PortName',         'XADC_DO_OUT', ...
    'PortWidth',        16, ...
    'InterfaceConnection',  'xadc_wiz_0/do_out');
hRD.addInternalIOInterface( ...
    'InterfaceID',      'Data ready signal (XADC DRP)', ...
    'InterfaceType',    'IN', ...
    'PortName',         'XADC_DRDY_OUT', ...
    'PortWidth',        1, ...
    'InterfaceConnection',  'xadc_wiz_0/drdy_out');
hRD.addInternalIOInterface( ...
    'InterfaceID',      'Address bus (XADC DRP)', ...
    'InterfaceType',    'OUT', ...
    'PortName',         'XADC_DADDR_IN', ...
    'PortWidth',        7, ...
    'InterfaceConnection',  'xadc_wiz_0/daddr_in');
hRD.addInternalIOInterface( ...
    'InterfaceID',      'Enable signal (XADC DRP)', ...
    'InterfaceType',    'OUT', ...
    'PortName',         'XADC_DEN_IN', ...
    'PortWidth',        1, ...
    'InterfaceConnection',  'xadc_wiz_0/den_in');

% LocalWords:  Zynq ZC vlnv xilinx zynq zc AXI axi Addr wiz aresetn IPCORE
% LocalWords:  avnet
