-- -------------------------------------------------------------
-- 
-- File Name: hdl_prj_ipcore2\hdlsrc\XADC_Single\xadc_ctrl.vhd
-- Created: 2016-06-02 23:32:07
-- 
-- Generated by MATLAB 8.6 and HDL Coder 3.7
-- 
-- -------------------------------------------------------------


-- -------------------------------------------------------------
-- 
-- Module: xadc_ctrl
-- Source Path: XADC_Single/xadc_reader_single/XADC_Com_Interface/xadc_ctrl
-- Hierarchy Level: 2
-- 
-- -------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE work.xadc_reader_single_pkg.ALL;

ENTITY xadc_ctrl IS
  PORT( clk                               :   IN    std_logic;
        reset                             :   IN    std_logic;
        enb                               :   IN    std_logic;
        drdy_i                            :   IN    std_logic;
        do_i                              :   IN    std_logic_vector(15 DOWNTO 0);  -- uint16
        den_o                             :   OUT   std_logic;
        daddr_o                           :   OUT   std_logic_vector(6 DOWNTO 0);  -- ufix7
        val_o                             :   OUT   vector_of_std_logic_vector16(0 TO 1)  -- uint16 [2]
        );
END xadc_ctrl;


ARCHITECTURE rtl OF xadc_ctrl IS

  -- Constants
  CONSTANT nc                             : vector_of_unsigned7(0 TO 1) := 
    (to_unsigned(16#00#, 7), to_unsigned(16#03#, 7));  -- ufix7 [2]

  -- Signals
  SIGNAL do_i_unsigned                    : unsigned(15 DOWNTO 0);  -- uint16
  SIGNAL daddr_o_tmp                      : unsigned(6 DOWNTO 0);  -- ufix7
  SIGNAL val_o_tmp                        : vector_of_unsigned16(0 TO 1);  -- uint16 [2]
  SIGNAL state                            : unsigned(1 DOWNTO 0);  -- ufix2
  SIGNAL val                              : vector_of_unsigned16(0 TO 1);  -- ufix16 [2]
  SIGNAL addr                             : unsigned(6 DOWNTO 0);  -- ufix7
  SIGNAL den                              : std_logic;
  SIGNAL addr_idx                         : unsigned(1 DOWNTO 0);  -- ufix2
  SIGNAL state_next                       : unsigned(1 DOWNTO 0);  -- ufix2
  SIGNAL val_next                         : vector_of_unsigned16(0 TO 1);  -- ufix16 [2]
  SIGNAL addr_next                        : unsigned(6 DOWNTO 0);  -- ufix7
  SIGNAL den_next                         : std_logic;
  SIGNAL addr_idx_next                    : unsigned(1 DOWNTO 0);  -- ufix2

BEGIN
  do_i_unsigned <= unsigned(do_i);

  xadc_ctrl_1_process : PROCESS (clk)
  BEGIN
    IF clk'EVENT AND clk = '1' THEN
      IF reset = '1' THEN
        state <= to_unsigned(16#0#, 2);
        val <= (OTHERS => to_unsigned(16#0000#, 16));
        addr <= to_unsigned(16#00#, 7);
        den <= '0';
        addr_idx <= to_unsigned(16#0#, 2);
      ELSIF enb = '1' THEN
        state <= state_next;
        val <= val_next;
        addr <= addr_next;
        den <= den_next;
        addr_idx <= addr_idx_next;
      END IF;
    END IF;
  END PROCESS xadc_ctrl_1_process;

  xadc_ctrl_1_output : PROCESS (drdy_i, do_i_unsigned, state, val, addr, den, addr_idx)
    VARIABLE sub_cast : signed(31 DOWNTO 0);
    VARIABLE sub_cast_0 : signed(31 DOWNTO 0);
  BEGIN
    state_next <= state;
    addr_next <= addr;
    den_next <= den;
    addr_idx_next <= addr_idx;
    --% Constants and defines
    -- Fixed point data types
    -- Address
    -- Value
    -- State
    -- The state machine will loop over the defined addresses
    -- reading out each value
    -- See Xilinx ug480 for more information
    -- % Value addresses
    -- Number of values to read
    -- Define the index data type based on the number of values
    -- index
    -- State machine defines
    -- Declare registers
    -- Set reset values
    --% Drive the outputs
    daddr_o_tmp <= addr;
    den_o <= den;
    val_next <= val;
    val_o_tmp <= val;
    --% Control the state
    CASE state IS
      WHEN "00" =>
        -- IDLE state, increment the index
        IF addr_idx = to_unsigned(16#2#, 2) THEN 
          addr_idx_next <= to_unsigned(16#1#, 2);
        ELSE 
          addr_idx_next <= addr_idx + to_unsigned(16#1#, 2);
        END IF;
        state_next <= to_unsigned(16#1#, 2);
      WHEN "01" =>
        -- ADDR state, write out the address
        sub_cast := signed(resize(addr_idx, 32));
        addr_next <= nc(to_integer(sub_cast - 1));
        den_next <= '0';
        state_next <= to_unsigned(16#2#, 2);
      WHEN "10" =>
        -- EN state, drive the enable high
        state_next <= to_unsigned(16#3#, 2);
        den_next <= '1';
      WHEN "11" =>
        -- WAIT state, wait for drdy
        IF drdy_i = '1' THEN 
          sub_cast_0 := signed(resize(addr_idx, 32));
          val_next(to_integer(sub_cast_0 - 1)) <= do_i_unsigned;
          state_next <= to_unsigned(16#0#, 2);
        END IF;
        den_next <= '0';
      WHEN OTHERS => 
        NULL;
    END CASE;
  END PROCESS xadc_ctrl_1_output;


  daddr_o <= std_logic_vector(daddr_o_tmp);

  outputgen: FOR k IN 0 TO 1 GENERATE
    val_o(k) <= std_logic_vector(val_o_tmp(k));
  END GENERATE;

END rtl;

